@@ -38,7 +38,7 @@ class HessianApproximation:
         :param hess:
             user provided initialization
         """
-        if hess is None:
+        if hess is None or not self.init_with_hess:
             self._hess = np.eye(dim)
         else:
             if hess.shape[0] != dim:
@@ -133,19 +133,6 @@ class DFP(Broyden):
         super(DFP, self).__init__(phi=1.0, init_with_hess=init_with_hess)
 
 
-class PSB(IterativeHessianApproximation):
-    """
-    Powell-symmetric-Broyden update strategy as introduced in
-    [Powell 1970](https://doi.org/10.1016/B978-0-12-597050-1.50006-3).
-    This is a rank 2 update strategy that preserves symmetry and
-    positive-semidefiniteness.
-
-    This scheme only works with a function that returns (fval, grad)
-    """
-    def update(self, s, y):
-        self._hess += broyden_class_update(y, s, self._hess, v=s)
-
-
 class SR1(IterativeHessianApproximation):
     """
     Symmetric Rank 1 update strategy as described in
@@ -212,9 +199,6 @@ class HybridSwitchApproximation(HessianApproximation):
     def get_mat(self) -> np.ndarray:
         return self.hessian_update.get_mat()
 
-    def set_mat(self, mat: np.ndarray):
-        self.hessian_update.set_mat(mat)
-
     def requires_hess(self):
         return True  # pragma: no cover
 
@@ -235,21 +219,18 @@ class HybridFixed(HybridSwitchApproximation):
             Iteration after which this approximation is used
         """
         self.switch_iteration: int = switch_iteration
-        self.iter: int = 0
         super(HybridFixed, self).__init__(happ)
+        self._switched = False
 
-    def init_mat(self, dim: int, hess: Optional[np.ndarray] = None):
-        self.iter = 0
-        super(HybridFixed, self).init_mat(dim, hess)
-        self._hess = hess
-
-    def update(self, s, y, hess):
+    def update(self, s, y, hess, iter_since_tr_update):
         self.hessian_update.update(s, y)
         self._hess = hess
-        self.iter += 1
+        if self._switched:
+            return
+        self._switched = iter_since_tr_update >= self.switch_iteration
 
     def get_mat(self) -> np.ndarray:
-        if self.iter >= self.switch_iteration:
+        if self._switched:
             return self.hessian_update.get_mat()
         else:
             return self._hess
@@ -258,9 +239,9 @@ class HybridFixed(HybridSwitchApproximation):
 class FX(HybridSwitchApproximation):
     def __init__(self,
                  happ: IterativeHessianApproximation = BFGS(),
-                 hybrid_tol: Optional[float] = 1e-2):
+                 hybrid_tol: Optional[float] = 0.2):
         r"""
-        Hybrid method as introduced by
+        Hybrid method HY2 as introduced by
         [Fletcher & Xu 1986](https://doi.org/10.1093/imanum/7.3.371). This
         approximation scheme employs a dynamic approximation as long as
         function values satisfy :math:`\frac{f_k - f_{k+1}}{f_k} < \epsilon`
@@ -275,59 +256,40 @@ class FX(HybridSwitchApproximation):
         self.hybrid_tol = hybrid_tol
         super(FX, self).__init__(happ)
 
-    def update(self, s, yb, r, rprev, hess):
-        yh = yb + hess.dot(s)
-        ratio = (norm(rprev)**2 - norm(r)**2)/(norm(rprev)**2)
+    def update(self, delta, gamma, r, rprev, hess):
+        # Equation (3.5)
+        ratio = (rprev.dot(rprev) - r.dot(r))/rprev.dot(rprev)
         if ratio >= self.hybrid_tol:
-            self.set_mat(hess)
+            self.hessian_update.set_mat(hess)
         else:
-            self.hessian_update.update(s, yh)
+            self.hessian_update.update(delta, gamma)
 
     def requires_resfun(self):
         return True  # pragma: no cover
 
 
-def _bfgs_vector(s, y, mat):
-    u = mat.dot(s)
-    c = u.T.dot(s)
-    b = y.T.dot(s)
-    rho = np.sqrt(b / c)
-    return y + np.sqrt(rho)*u
-
-
-def _psb_vector(s, y, mat):
-    return s
-
-
-def _dfp_vector(s, y, mat):
-    return y
-
-
 class StructuredApproximation(HessianApproximation):
-    vector_routines = {
-        'BFGS': _bfgs_vector,
-        'PSB': _psb_vector,
-        'DFP': _dfp_vector,
-    }
-
-    def __init__(self,
-                 update_method: Optional[str] = 'BFGS'):
+    def __init__(self, phi: Optional[float] = 0.0):
         """
         This is the base class for structured secant methods (SSM). SSMs
         approximate the hessian by combining the Gauss-Newton component C(x)
         and an iteratively updated component that approximates the
         difference S to the true Hessian.
+
+        :parameter phi:
+            convex combination parameter interpolating between BFGS (phi==0)
+            and DFP (phi==1) update schemes.
         """
         self.A: np.ndarray = np.empty(0)
-        if update_method not in self.vector_routines:
-            raise ValueError(f'Unknown update method {update_method}. Known '
-                             f'methods are {self.vector_routines.keys()}')
-
-        self.vector_routine = self.vector_routines[update_method]
-        super(StructuredApproximation, self).__init__()
+        self.phi = phi
+        if phi < 0 or phi > 1:
+            warnings.warn('Setting phi to values outside the interval [0, 1]'
+                          'will not guarantee that positive definiteness is '
+                          'preserved during updating.')
+        super(StructuredApproximation, self).__init__(init_with_hess=True)
 
     def init_mat(self, dim: int, hess: Optional[np.ndarray] = None):
-        self.A = np.zeros((dim, dim))
+        self.A = np.eye(dim) * np.spacing(1)
         super(StructuredApproximation, self).init_mat(dim, hess)
 
     def update(self, s: np.ndarray, y: np.ndarray, r: np.ndarray,
@@ -345,18 +307,21 @@ class SSM(StructuredApproximation):
     """
     Structured Secant Method as introduced by
     [Dennis et al 1989](https://doi.org/10.1007/BF00962795), which is
-    compatible with BFGS, DFP and PSB update schemes.
+    compatible with BFGS, DFP update schemes.
 
     This scheme only works with a function that returns (res, sres)
     """
 
     def update(self, s: np.ndarray, y: np.ndarray, r: np.ndarray,
                hess: np.ndarray, yb: np.ndarray):
-        Bs = hess + self.A  # Bs = A + C(x)
-        ys = yb + hess.dot(s)  # ys = y# + C(x)*s
-        v = self.vector_routine(s, ys, Bs)
-        self._hess = Bs + broyden_class_update(s, ys, Bs, v=v)
-        self.A += broyden_class_update(s, yb, self.A, v=v)
+        # B^S = A + C(x_+)
+        Bs = hess + self.A
+        # y^S = y^# + C(x_+)*s
+        ys = yb + hess.dot(s)
+        # Equation (13)
+        self.A += broyden_class_update(ys, s, Bs, phi=self.phi)
+        # B_+ = C(x_+) + A + BFGS update A (=A_+)
+        self._hess = hess + self.A
 
 
 class TSSM(StructuredApproximation):
@@ -370,9 +335,13 @@ class TSSM(StructuredApproximation):
 
     def update(self, s: np.ndarray, y: np.ndarray, r: np.ndarray,
                hess: np.ndarray, yb: np.ndarray):
+        # Equation (2.7)
         Bs = hess + norm(r) * self.A
-        v = self.vector_routine(s, y, Bs)
-        self.A += broyden_class_update(s, yb, self.A, v=v)
+        # Equation (2.6)
+        ys = hess.dot(s) + yb
+        # Equation (2.10)
+        self.A += broyden_class_update(ys, s, Bs, phi=self.phi)/norm(r)
+        # Equation (2.9)
         self._hess = hess + norm(r) * self.A
 
 
@@ -390,24 +359,25 @@ class GNSBFGS(StructuredApproximation):
             switching tolerance that controls switching between update methods
         """
         self.hybrid_tol: float = hybrid_tol
-        super(GNSBFGS, self).__init__('BFGS')
+        super(GNSBFGS, self).__init__(phi=0.0)
 
     def update(self, s: np.ndarray, y: np.ndarray, r: np.ndarray,
                hess: np.ndarray, yb: np.ndarray):
+        # Equation (2.1)
         ratio = yb.T.dot(s)/s.dot(s)
         if ratio > self.hybrid_tol:
-            self.A += broyden_class_update(s, yb, self.A, phi=1.0)
+            # Equation (2.3)
+            self.A += broyden_class_update(yb, s, self.A, phi=self.phi)
+            # Equation (2.2)
             self._hess = hess + self.A
         else:
+            # Equation (2.2)
             self._hess = hess + norm(r) * np.eye(len(y))
 
 
-def broyden_class_update(y, s, mat, phi=None, v=None):
+def broyden_class_update(y, s, mat, phi=0.0):
     """
-    Scale free implementation of the broyden class update scheme. This can
-    either be called by using a phi parameter that interpolates between BFGS
-    (phi=0) and DFP (phi=1) or by using the weighting vector v that allows
-    implementation of PSB (v=s), DFP (v=y) and BFGS (V=y+rho*B*s).
+    Scale free implementation of the broyden class update scheme.
 
     :param y:
         difference in gradient
@@ -416,10 +386,8 @@ def broyden_class_update(y, s, mat, phi=None, v=None):
     :param mat:
         current hessian approximation
     :param phi:
-        convex combination parameter. Must not pass this parameter at the same
-        time as v.
-    :param v:
-        weighting vector. Must not pass this parameter at the same time as phi.
+        convex combination parameter, interpolates between BFGS (phi=0) and
+        DFP (phi=1).
     """
     u = mat.dot(s)
     c = u.T.dot(s)
@@ -428,25 +396,10 @@ def broyden_class_update(y, s, mat, phi=None, v=None):
     if b <= 0:
         return np.zeros(mat.shape)
 
-    if v is None and phi is not None:
-        bfgs = phi == 0
-    elif v is not None and phi is None:
-        bfgs = False
-    else:
-        raise ValueError('Exactly one of the values of phi and v must be '
-                         'provided.')
+    update = np.outer(y, y.T) / b - np.outer(u, u.T) / c
 
-    if bfgs:  # BFGS
-        return np.outer(y, y.T) / b - np.outer(u, u.T) / c
-
-    if v is None:
-        rho = np.sqrt(b / c) if c > 0 else 0  # c == 0 iff u == 0
-        v = y + (1-phi) * rho * u
-
-    z = y - mat.dot(s)
-    d = v.T.dot(s)
-    a = z.T.dot(s) / (d ** 2)
-
-    update = (np.outer(z, v.T) + np.outer(v, z.T)) / d - a * np.outer(v, v.T)
+    if phi != 0.0:
+        v = y / b - u / c
+        update += phi * c * np.outer(v, v.T)
 
     return update
